# coding=utf-8
from unittest import TestCase
import os
from tempfile import mktemp

import numpy as np
# import dummydata
import iris

from earthdiagnostics.diagnostic import DiagnosticVariableListOption, DiagnosticOptionError
from earthdiagnostics.box import Box
from earthdiagnostics.general.select_levels import SelectLevels
from earthdiagnostics.frequency import Frequencies
from mock import Mock, patch

from earthdiagnostics.modelingrealm import ModelingRealms


class TestSelectLevels(TestCase):

    def setUp(self):
        self.data_manager = Mock()

        self.diags = Mock()
        self.diags.config.experiment.get_chunk_list.return_value = (('20010101', 0, 0), ('20010101', 0, 1))
        self.diags.config.experiment.startdates = ['20010101', ]
        self.diags.config.frequency = Frequencies.monthly

        self.box = Box()
        self.box.min_depth = 0
        self.box.max_depth = 100
        self.var_file = mktemp('.nc')

    def tearDown(self):
        if os.path.exists(self.var_file):
            os.remove(self.var_file)

    def fake_parse(self, value):
        return value.split('-')

    @patch.object(DiagnosticVariableListOption, 'parse', fake_parse)
    def test_generate_jobs(self):
        jobs = SelectLevels.generate_jobs(self.diags, ['diagnostic', 'atmos', 'var', '0', '20'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], SelectLevels(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var',
                                               '', 0, 20))
        self.assertEqual(jobs[1], SelectLevels(self.data_manager, '20010101', 0, 1, ModelingRealms.atmos, 'var',
                                               '', 0, 20))

        jobs = SelectLevels.generate_jobs(self.diags, ['diagnostic', 'atmos', 'var1-var2', '0', '20'])
        self.assertEqual(len(jobs), 4)
        self.assertEqual(jobs[0], SelectLevels(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var1',
                                               '', 0, 20))
        self.assertEqual(jobs[1], SelectLevels(self.data_manager, '20010101', 0, 1, ModelingRealms.atmos, 'var1',
                                               '', 0, 20))
        self.assertEqual(jobs[2], SelectLevels(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var2',
                                               '', 0, 20))
        self.assertEqual(jobs[3], SelectLevels(self.data_manager, '20010101', 0, 1, ModelingRealms.atmos, 'var2',
                                               '', 0, 20))

        jobs = SelectLevels.generate_jobs(self.diags, ['diagnostic', 'atmos', 'var', '0', '20', 'grid'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], SelectLevels(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var',
                                               'grid', 0, 20))
        self.assertEqual(jobs[1], SelectLevels(self.data_manager, '20010101', 0, 1, ModelingRealms.atmos, 'var',
                                               'grid', 0, 20))

        with self.assertRaises(DiagnosticOptionError):
            SelectLevels.generate_jobs(self.diags, ['diagnostic'])

            with self.assertRaises(DiagnosticOptionError):
                SelectLevels.generate_jobs(self.diags, ['diagnostic', 'atmos', 'var', '0', '20', 'grid', 'extra'])

    def test_str(self):
        select = SelectLevels(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var', 'grid', 0, 20)
        self.assertEqual(str(select),
                         'Select levels Startdate: 20010101 Member: 0 Chunk: 0 Variable: atmos:var '
                         'Levels: 0-20 Grid: grid')

    # def test_compute(self):
    #     dummydata.model3.Model3(oname=self.var_file, var='ta', start_year=2000, stop_year=2000, method='constant',
    #                             constant=1)
    #     select = SelectLevels(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'ta', 'grid', 0, 3)
    #     select.variable_file = Mock()
    #     select.variable_file.local_file = self.var_file

    #     select.result = Mock()
    #     select.compute()
    #     select.result.set_local_file.assert_called_once()
    #     cube = iris.load_cube(select.result.set_local_file.call_args[0][0])
    #     original_cube = iris.load_cube(select.result.set_local_file.call_args[0][0])
    #     self.assertTrue(np.all(cube.coord('air_pressure').points == original_cube.coord('air_pressure').points[0:4]))
