# coding=utf-8
from unittest import TestCase
from tempfile import mktemp
from mock import Mock, patch
import os
# import dummydata

from earthdiagnostics.diagnostic import DiagnosticVariableOption
from earthdiagnostics.frequency import Frequencies
from earthdiagnostics.general.timemean import YearlyMean
from earthdiagnostics.modelingrealm import ModelingRealms


class TestYearlyMean(TestCase):

    def setUp(self):
        self.var_file = mktemp('.nc')
        self.data_manager = Mock()

        self.diags = Mock()
        self.diags.config.experiment.get_chunk_list.return_value = (('20010101', 0, 0), ('20010101', 0, 1))
        self.diags.config.frequency = Frequencies.monthly

        self.yearly_mean = YearlyMean(self.data_manager, '20000101', 1, 1, ModelingRealms.ocean, 'var', 'freq', '')

    def tearDown(self):
        if os.path.exists(self.var_file):
            os.remove(self.var_file)

    def fake_parse(self, value):
        return value

    @patch.object(DiagnosticVariableOption, 'parse', fake_parse)
    def test_generate_jobs(self):
        jobs = YearlyMean.generate_jobs(self.diags, ['diagnostic', 'atmos', 'var', 'day'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], YearlyMean(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var',
                                             Frequencies.daily, ''))
        self.assertEqual(jobs[1], YearlyMean(self.data_manager, '20010101', 0, 1, ModelingRealms.atmos, 'var',
                                             Frequencies.daily, ''))

        jobs = YearlyMean.generate_jobs(self.diags, ['diagnostic', 'seaice', 'var', 'mon', 'grid'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], YearlyMean(self.data_manager, '20010101', 0, 0, ModelingRealms.seaIce, 'var',
                                             Frequencies.monthly, 'grid'))
        self.assertEqual(jobs[1], YearlyMean(self.data_manager, '20010101', 0, 1, ModelingRealms.seaIce, 'var',
                                             Frequencies.monthly, 'grid'))

        with self.assertRaises(Exception):
            YearlyMean.generate_jobs(self.diags, ['diagnostic'])

        with self.assertRaises(Exception):
            YearlyMean.generate_jobs(self.diags, ['diagnostic', '0', '0', '0', '0', '0', '0', '0'])

    def test_str(self):
        self.assertEqual(str(self.yearly_mean),
                         'Calculate yearly mean Startdate: 20000101 Member: 1 Chunk: 1 '
                         'Variable: ocean:var Original frequency: freq Grid: ')

    # def test_compute(self):
    #     dummydata.model3.Model3(oname=self.var_file, start_year=2000, stop_year=2000, method='constant', constant=1)

    #     self.yearly_mean.variable_file = Mock()
    #     self.yearly_mean.variable_file.local_file = self.var_file
    #     self.yearly_mean.mean_file = Mock()
    #     self.yearly_mean.compute()
    #     self.yearly_mean.mean_file.set_local_file.assert_called_once()
