# coding=utf-8
from unittest import TestCase
from mock import Mock

from earthdiagnostics.constants import Basin, Basins


class TestBasin(TestCase):

    def setUp(self):
        self.basin = Basin('Basin')

    def test_name(self):
        self.assertEqual('Basin', self.basin.name)

    def test__eq__(self):
        self.assertTrue(Basin('Basin') == self.basin)
        self.assertFalse(Basin('bas') == self.basin)

    def test__neq__(self):
        self.assertFalse(Basin('Basin') != self.basin)
        self.assertTrue(Basin('bas') != self.basin)

    def test__greater(self):
        self.assertFalse(Basin('Aasin') > self.basin)
        self.assertFalse(Basin('Basin') > self.basin)
        self.assertTrue(Basin('Casin') > self.basin)

    def test__lower(self):
        self.assertTrue(Basin('Aasin') < self.basin)
        self.assertFalse(Basin('Basin') < self.basin)
        self.assertFalse(Basin('Casin') < self.basin)

    def test_ge(self):
        self.assertFalse(Basin('Aasin') >= self.basin)
        self.assertTrue(Basin('Basin') >= self.basin)
        self.assertTrue(Basin('Casin') >= self.basin)

    def test__le(self):
        self.assertTrue(Basin('Aasin') <= self.basin)
        self.assertTrue(Basin('Basin') <= self.basin)
        self.assertFalse(Basin('Casin') <= self.basin)

    def test__str__(self):
        self.assertEqual(str(self.basin), 'Basin')

    def test__repr__(self):
        self.assertEqual(repr(self.basin), 'Basin')

    def test_order(self):
        self.assertTrue(self.basin < Basin('Vasin'))
        self.assertTrue(self.basin > Basin('Asin'))


class TestBasins(TestCase):

    def test_singleton(self):
        self.assertIs(Basins(), Basins())

    def test_get_basins(self):
        mock_handler = Mock()
        mock_handler.variables.keys.return_value = ('var1', 'lat')
        Basins().get_available_basins(mock_handler)
        self.assertTrue(hasattr(Basins(), 'var1'))
        self.assertFalse(hasattr(Basins(), 'lat'))

    def test_get_basins_with_alias(self):
        mock_handler = Mock()
        mock_handler.variables.keys.return_value = ('Kara_Sea', 'lat')
        basins = Basins()
        Basins().get_available_basins(mock_handler)
        self.assertTrue(hasattr(basins, 'Kara_Sea'))
        self.assertFalse(hasattr(basins, 'lat'))
