# coding=utf-8


from unittest import TestCase
from mock import Mock

from earthdiagnostics.datamanager import DataManager
from earthdiagnostics.modelingrealm import ModelingRealms


class TestDataManager(TestCase):

    def setUp(self):
        self.config = Mock()
        self.config.experiment.atmos_timestep = 3
        self.config.experiment.ocean_timestep = 6

        self.data_manager = DataManager(self.config)

    def test_prepare(self):
        self.data_manager.prepare()

    def test_link_file(self):
        self.data_manager.link_file(None, '', None, '', 0, 0)

    def test_file_exists(self):
        with self.assertRaises(NotImplementedError):
            self.data_manager.file_exists(None, '', '', 0, 0)

    def test_request_chunk(self):
        with self.assertRaises(NotImplementedError):
            self.data_manager.request_chunk(None, '', '', 0, 0)

    def test_request_year(self):
        with self.assertRaises(NotImplementedError):
            self.data_manager.request_year(None, '', '', '', 0, 0)

    def test_declare_chunk(self):
        with self.assertRaises(NotImplementedError):
            self.data_manager.declare_chunk(None, '', '', 0, 0)

    def test_declare_year(self):
        with self.assertRaises(NotImplementedError):
            self.data_manager.declare_year(None, '', '', 0, 0)

    def test_get_final_varname(self):
        self.assertEqual(self.data_manager._get_final_var_name(None, 'var'), 'var')
        box = Mock()
        box.get_lon_str.return_value = '_lon'
        box.get_lat_str.return_value = '_lat'
        box.get_depth_str.return_value = '_depth'
        self.assertEqual(self.data_manager._get_final_var_name(box, 'var'), 'var_lon_lat_depth')
